/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TRAVERSAL_CALLABLE_TRAITS_20070926_1427)
#define BOOST_TRAVERSAL_CALLABLE_TRAITS_20070926_1427

#include <boost/function_types/function_arity.hpp>
#include <boost/function_types/result_type.hpp>
#include <boost/function_types/parameter_types.hpp>

#include <boost/mpl/has_xxx.hpp>
#include <boost/mpl/eval_if.hpp>
#include <boost/mpl/or.hpp>
#include <boost/mpl/int.hpp>
#include <boost/mpl/bool.hpp>

#include <boost/static_assert.hpp>

namespace boost { namespace traversal { namespace detail {

    BOOST_MPL_HAS_XXX_TRAIT_DEF(third_argument_type);
    BOOST_MPL_HAS_XXX_TRAIT_DEF(second_argument_type);
    BOOST_MPL_HAS_XXX_TRAIT_DEF(argument_type);
    BOOST_MPL_HAS_XXX_TRAIT_DEF(result_type);

    template<typename T>
    struct is_mono_function_object
        : mpl::or_<
        has_argument_type<T>,
        has_second_argument_type<T>,
        has_third_argument_type<T> >
    {};

    template<typename T>
    struct mono_result_type
    {
        typedef typename T::result_type type;
    };

    template<typename T>
    struct mono_arity
    {
        typedef typename mpl::eval_if<
            has_third_argument_type<T>,
            mpl::int_<3>,
            mpl::eval_if<
            has_second_argument_type<T>,
            mpl::int_<2>,
            mpl::int_<1> > >::type type;
        BOOST_STATIC_CONSTANT(int, value = type::value);
    };

    template<int N, typename T>
    struct mono_argument_c
    {
        BOOST_STATIC_ASSERT(N > 0 && N < 4);
    };

    template<typename T>
    struct extract_argument_type
    {
        typedef typename T::argument_type type;
    };

    template<typename T>
    struct extract_first_argument_type
    {
        typedef typename T::first_argument_type type;
    };

    template<typename T>
    struct mono_argument_c<0, T>
        : mpl::eval_if<
        mpl::bool_<mono_arity<T>::value == 1>,
        extract_argument_type<T>,
        extract_first_argument_type<T> >
    {};

    template<typename T>
    struct mono_argument_c<1, T>
    {
        typedef typename T::second_argument_type type;
    };

    template<typename T>
    struct mono_argument_c<2, T>
    {
        typedef typename T::third_argument_type type;
    };

    template<typename T>
    struct callable_arity
    {
        typedef typename mpl::eval_if<
            is_mono_function_object<T>,
            mono_arity<T>,
            function_types::function_arity<T> >::type type;
        BOOST_STATIC_CONSTANT(int, value = type::value);
    };

    template<typename T>
    struct callable_result_type
        : mpl::eval_if<
        is_mono_function_object<T>,
        mono_result_type<T>,
        function_types::result_type<T> >
    {};

    template<int N, typename T>
    struct extract_function_argument_c
        : mpl::at_c<typename function_types::parameter_types<T>::type, N>
    {};

    template<int N, typename T>
    struct callable_argument_c
        : mpl::eval_if<
        is_mono_function_object<T>,
        mono_argument_c<N, T>,
        extract_function_argument_c<N, T> >
    {};
}}}

#endif
