/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TRAVERSAL_20070926_1001)
#define BOOST_TRAVERSAL_20070926_1001

namespace boost { namespace traversal {
    namespace detail
    {
        template<typename Sig, typename Func>
        struct restrict;

        template<typename Ret, typename Arg0, typename Func>
        struct restrict<Ret(Arg0), Func>
        {
            typedef Ret result_type;

            explicit restrict(Func f)
                : f_(f), ret_()
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f), ret_(ret)
                {}

            Ret operator()(Arg0 a0) const
                {
                    return f_(a0);
                }

            template<typename U0>
            Ret operator()(U0& arg0) const
                {
                    return ret_;
                }

            Func f_;
            Ret ret_;
        };

        template<typename Arg0, typename Func>
        struct restrict<void(Arg0), Func>
        {
            typedef void result_type;

            explicit restrict(Func f)
                : f_(f)
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f)
                {}

            void operator()(Arg0 a0) const
                {
                    f_(a0);
                }

            template<typename U0>
            void operator()(U0& arg0) const
                {}

            Func f_;
        };

        template<typename Ret, typename Arg0, typename Arg1, typename Func>
        struct restrict<Ret(Arg0,Arg1), Func>
        {
            typedef Ret result_type;

            explicit restrict(Func f)
                : f_(f), ret_()
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f), ret_(ret)
                {}

            Ret operator()(Arg0 a0, Arg1 a1) const
                {
                    return f_(a0, a1);
                }

            template<typename U0, typename U1>
            Ret operator()(U0& a0, U1& a1) const
                {
                    return ret_;
                }

            Func f_;
            Ret ret_;
        };

        template<typename Arg0, typename Arg1, typename Func>
        struct restrict<void(Arg0,Arg1), Func>
        {
            typedef void result_type;

            explicit restrict(Func f)
                : f_(f)
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f)
                {}

            void operator()(Arg0 a0, Arg1 a1) const
                {
                    f_(a0, a1);
                }

            template<typename U0, typename U1>
            void operator()(U0& a0, U1& a1) const
                {}

            Func f_;
        };

        template<typename Ret, typename Arg0, typename Arg1, typename Arg2, typename Func>
        struct restrict<Ret(Arg0,Arg1,Arg2), Func>
        {
            typedef Ret result_type;

            explicit restrict(Func f)
                : f_(f), ret_()
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f), ret_(ret)
                {}

            Ret operator()(Arg0 a0, Arg1 a1, Arg2 a2) const
                {
                    return f_(a0, a1, a2);
                }

            template<typename U0, typename U1, typename U2>
            Ret operator()(U0& a0, U1& a1, U2& a2) const
                {
                    return ret_;
                }

            Func f_;
            Ret ret_;
        };

        template<typename Arg0, typename Arg1, typename Arg2, typename Func>
        struct restrict<void(Arg0,Arg1,Arg2), Func>
        {
            typedef void result_type;

            explicit restrict(Func f)
                : f_(f)
                {}

            template<typename R>
            restrict(Func f, R ret)
                : f_(f)
                {}

            void operator()(Arg0 a0, Arg1 a1, Arg2 a2) const
                {
                    f_(a0, a1, a2);
                }

            template<typename U0, typename U1, typename U2>
            void operator()(U0& a0, U1& a1, U2& a2) const
                {}

            Func f_;
        };
    }

    template<typename Sig, typename F>
    detail::restrict<Sig, F>
    restrict(F f)
    {
        return detail::restrict<Sig, F>(f);
    }

    template<typename Sig, typename F, typename Dflt>
    detail::restrict<Sig, F>
    restrict(F f, Dflt dflt)
    {
        return detail::restrict<Sig, F>(f, dflt);
    }
}}

#endif
