/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TRAVERSE_TRAVERSE_20070922_1846)
#define BOOST_TRAVERSE_TRAVERSE_20070922_1846

#include <boost/traversal/horizontal/one.hpp>

namespace boost { namespace traversal {
    namespace detail
    {
        template<typename InUnaryFunc, typename OutUnaryFunc, typename OneImplementation, typename Combiner>
        struct traverse_traversal_impl
        {
            traverse_traversal_impl(InUnaryFunc in, OutUnaryFunc out, OneImplementation one, Combiner combiner)
                : in_(in), out_(out), one_(one), combiner_(combiner)
            {}

            template<typename T>
            bool operator()(T& t) const
            {
                return combiner_(t, in_, out_, one_, *this);
            }
            
            InUnaryFunc in_;
            OutUnaryFunc out_;
            OneImplementation one_;
            Combiner combiner_;
        };
    }

    struct traverse_traversal
    {
        typedef bool result_type;

        template<typename T, typename InUnaryFunc, typename OutUnaryFunc, typename Combiner, typename OneImplementation>
        bool operator()(T& t, InUnaryFunc in, OutUnaryFunc out, OneImplementation one, Combiner combiner) const
        {
            detail::traverse_traversal_impl<
                InUnaryFunc, 
                OutUnaryFunc, 
                OneImplementation, 
                Combiner> tr(in, out, one, combiner);
            return tr(t);
        }
    };

    traverse_traversal const traverse = {};
}}

#endif
