/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TRAVERSAL_TRAVERSE_PARENT_20070930_1701)
#define BOOST_TRAVERSAL_TRAVERSE_PARENT_20070930_1701

#include <boost/traversal/recursive/detail/no_parent.hpp>

namespace boost { namespace traversal {
    namespace detail
    {
        template<typename InUnaryFunc, typename OutUnaryFunc, typename OneImplementation, 
                 typename Combiner, typename Parent>
        struct traverse_parent_traversal_impl
        {
            traverse_parent_traversal_impl(InUnaryFunc in, OutUnaryFunc out, OneImplementation one, 
                                           Combiner combiner, Parent& parent)
                : in_(in), out_(out), one_(one), combiner_(combiner), parent_(parent)
            {}

            template<typename T>
            bool operator()(T& t) const
            {
                return combiner_(t, in_, out_, one_, *this, parent_);
            }

            template<typename NewParent>
            traverse_parent_traversal_impl<InUnaryFunc, OutUnaryFunc, OneImplementation,
                                           Combiner, NewParent>
            rebind(NewParent& new_parent) const
            {
                return traverse_parent_traversal_impl<InUnaryFunc, OutUnaryFunc, OneImplementation,
                    Combiner, NewParent>(in_, out_, one_, combiner_, new_parent);
            }
            
            InUnaryFunc in_;
            OutUnaryFunc out_;
            OneImplementation one_;
            Combiner combiner_;
            Parent& parent_;
        };
    }

    struct traverse_parent_traversal
    {
        typedef bool result_type;

        template<typename T, typename InUnaryFunc, typename OutUnaryFunc, typename Combiner, typename OneImplementation>
        bool operator()(T& t, InUnaryFunc in, OutUnaryFunc out, OneImplementation one, Combiner combiner) const
        {
            detail::NoParent no_parent;
            detail::traverse_parent_traversal_impl<
                InUnaryFunc, 
                OutUnaryFunc, 
                OneImplementation, 
                Combiner,
                detail::NoParent> tr(in, out, one, combiner, no_parent);
            return tr(t);
        }
    };

    traverse_parent_traversal const traverse_parent = {};
}}

#endif
