/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TU_TRAVERSE_PARENT_TU_TRAVERSE_PARENT_20070922_1846)
#define BOOST_TU_TRAVERSE_PARENT_TU_TRAVERSE_PARENT_20070922_1846

#include <boost/traversal/recursive/detail/no_parent.hpp>

namespace boost { namespace traversal {
    namespace detail
    {
        template<typename InUnaryFunc, typename OutUnaryFunc, typename OneImplementation, 
                 typename TUCombiner, typename Parent>
        struct tu_traverse_parent_traversal_impl
        {
            tu_traverse_parent_traversal_impl(InUnaryFunc in, OutUnaryFunc out, OneImplementation one, 
                                              TUCombiner tu_combiner, Parent& parent)
                : in_(in), out_(out), one_(one), tu_combiner_(tu_combiner), parent_(parent)
            {}

            template<typename State, typename T>
            State operator()(State state, T& t) const
            {
                return tu_combiner_(t, state, in_, out_, one_, *this, parent_);
            }

            template<typename NewParent>
            tu_traverse_parent_traversal_impl<InUnaryFunc, OutUnaryFunc, OneImplementation,
                                           TUCombiner, NewParent>
            rebind(NewParent& new_parent) const
            {
                return tu_traverse_parent_traversal_impl<InUnaryFunc, OutUnaryFunc, OneImplementation,
                    TUCombiner, NewParent>(in_, out_, one_, tu_combiner_, new_parent);
            }
            
            InUnaryFunc in_;
            OutUnaryFunc out_;
            OneImplementation one_;
            TUCombiner tu_combiner_;
            Parent parent_;
        };
    }

    struct tu_traverse_parent_traversal
    {
        template<typename Sig>
        struct result;

        template<typename T, typename State>
        struct result<tu_traverse_parent_traversal(T&, State)>
        {
            typedef std::pair<State, bool> type;
        };

        template<typename T, typename State, typename InUnaryFunc, typename OutUnaryFunc, 
                 typename TUCombiner, typename OneImplementation>
        std::pair<State, bool>
        operator()(T& t, State state, InUnaryFunc in, OutUnaryFunc out, 
                        OneImplementation one, TUCombiner tu_combiner) const
        {
            detail::NoParent no_parent;
            detail::tu_traverse_parent_traversal_impl<
                InUnaryFunc, 
                OutUnaryFunc, 
                OneImplementation, 
                TUCombiner,
                detail::NoParent> tr(in, out, one, tu_combiner, no_parent);
            return tr(std::make_pair(state, false), t);
        }
    };

    tu_traverse_parent_traversal const tu_traverse_parent = {};
}}

#endif
