/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_TRAVERSAL_IS_POINTER_20070901)
#define BOOST_TRAVERSAL_IS_POINTER_20070901

#include <boost/mpl/bool.hpp>
#include <boost/mpl/or.hpp>
#include <boost/mpl/and.hpp>
#include <boost/mpl/not.hpp>

#include <boost/type_traits/is_pointer.hpp>
#include <boost/type_traits/is_function.hpp>
#include <boost/type_traits/remove_pointer.hpp>

#include <memory>

namespace boost { 

#define BOOST_DECL_PTR(name)                    \
    template<typename T>                        \
    class name;

    BOOST_DECL_PTR(shared_ptr);
    BOOST_DECL_PTR(scoped_ptr);
    BOOST_DECL_PTR(scoped_array);
    BOOST_DECL_PTR(shared_array);
    BOOST_DECL_PTR(weak_ptr);
    BOOST_DECL_PTR(intrusive_ptr);

#undef BOOST_DECL_PTR


namespace traversal {
    namespace traits
    {
        namespace detail
        {
            template<typename T>
            struct is_object_pointer
                : mpl::and_<
                boost::is_pointer<T>,
                mpl::not_<boost::is_function<typename boost::remove_pointer<T>::type> > >
            {};
        }

        template<typename T>
        struct is_smart_pointer
            : mpl::false_
        {};

#define PTR_TRAITS(ns, name)                    \
        template<typename T>                    \
        struct is_smart_pointer<ns::name<T> >   \
            : mpl::true_                        \
        {}

        PTR_TRAITS(std, auto_ptr);

#define BOOST_PTR_TRAITS(name) PTR_TRAITS(boost, name)

        BOOST_PTR_TRAITS(shared_ptr);
        BOOST_PTR_TRAITS(scoped_ptr);
        BOOST_PTR_TRAITS(scoped_array);
        BOOST_PTR_TRAITS(shared_array);
        BOOST_PTR_TRAITS(weak_ptr);
        BOOST_PTR_TRAITS(intrusive_ptr);

#undef BOOST_PTR_TRAITS
#undef PTR_TRAITS

        template<typename T>
        struct is_pointer
            : mpl::or_<
            detail::is_object_pointer<T>,
            is_smart_pointer<T> >
        {};
    }
}}

#endif
