/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/

/*=============================================================================
  Simple example:
  Create a nested set of standard containers.
  Do some computations on the containers without needing to write any
  traversal code.
==============================================================================*/

#include <boost/traversal/recursive/tu_full.hpp>
#include <boost/traversal/recursive/full.hpp>
#include <boost/traversal/functional/extend.hpp>

#include <vector>
#include <list>
#include <deque>
#include <cassert>

using namespace std;
using namespace boost;

namespace
{
    pair<int, bool> 
    accumulate_ints(pair<int, bool> state, int new_value)
    {
        // State is the value being built up, and a termination flag
        state.first += new_value;
        return state;
    }

    bool increment_ints(int& i)
    {
        ++i;
        // Return value is a termination flag
        return false;
    }
}

int main()
{
    // Generate a large, if somewhat unrealistic nested std container type
    typedef vector<int> int_vec;
    typedef deque<int_vec> int_vec_deq;
    typedef list<int_vec_deq> int_vec_deq_lst;

    // Fill up with some test data
    int_vec_deq d1;
    d1.push_back(int_vec(1, 101));
    d1.push_back(int_vec(2, 202));
    int_vec_deq d2;
    d2.push_back(int_vec(3, 303));

    int_vec_deq_lst l;
    l.push_back(d1);
    l.push_back(d2);

    // Add up all the integer values in the structure
    int const sum_of_all_integers = 
        traversal::tu_full(l, 0, traversal::extend(accumulate_ints)).first;

    assert(sum_of_all_integers == 1 * 101 + 2 * 202 + 3 * 303);

    // Increment all the integer values by one
    traversal::full(l, traversal::extend(increment_ints));

    // Check all the new total is as expected
    assert(traversal::tu_full(l, 0, traversal::extend(accumulate_ints)).first
           == 1 * (101 + 1) + 2 * (202 + 1) + 3 * (303 + 1));
}
