/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#include <boost/traversal/recursive/bu_once.hpp>
#include <boost/detail/lightweight_test.hpp>

#include <vector>
#include <boost/fusion/container/vector/vector10.hpp>
#include <boost/fusion/sequence/intrinsic/at.hpp>

using namespace boost;

namespace
{
    struct increment_ints_bu_once
    {
        bool operator()(int& i) const
        {
            ++i;
            return true;
        }

        template<typename T>
        bool operator()(T& t) const
        {
            return false;
        }
    };
}

int main()
{
    std::vector<int> std_v(2, 101);
    traversal::bu_once(std_v, increment_ints_bu_once());
    BOOST_TEST(std_v[0] == 102);
    BOOST_TEST(std_v[1] == 101);

    fusion::vector2<int, int> fusion_v(101, 202);
    traversal::bu_once(fusion_v, increment_ints_bu_once());
    BOOST_TEST(fusion::at_c<0>(fusion_v) == 102);
    BOOST_TEST(fusion::at_c<1>(fusion_v) == 202);

    return boost::report_errors();
}
