/*=============================================================================
    Copyright (c) 2007-2008 Dan Marsden

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#include <boost/traversal/recursive/stop.hpp>

#include <boost/detail/lightweight_test.hpp>

#include <vector>

using namespace boost;

namespace
{
    // Skip vector<int> if longer than 1 element
    struct stop_function
    {
        template<typename T>
        bool operator()(T&) const
        {
            return false;
        }

        bool operator()(std::vector<int>& v) const
        {
            return v.size() > 1;
        }

        bool operator()(int& i) const
        {
            ++i;
            return true; // Test this will not stop horizontal iteration
        }
    };
}

int main()
{
    std::vector<std::vector<int> > v;
    v.push_back(std::vector<int>(1, 0));
    v.push_back(std::vector<int>(2, 0));

    traversal::stop(v, stop_function());

    BOOST_TEST(v[0][0] == 1);
    BOOST_TEST(v[1][0] == 0);
    
    return boost::report_errors();
}
